#!/usr/bin/env python
#
# Write IPSet alerts to IP set files.
# ------------------------------------------------------------------------
# Copyright (C) 2012 Carnegie Mellon University. All Rights Reserved.
# ------------------------------------------------------------------------
# Authors: Tony Cebzanov <tonyc@cert.org>
# -----------------------------------------------------------------------------
# GNU Public License (GPL) Rights pursuant to Version 2, June 1991
# Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
# -----------------------------------------------------------------------------
#
# Sample Usage:
#
#     snarf2ipset.py name_of_field


import sys
from snarf import *
from snarf.alert import datetime_from_timestamp
from optparse import OptionParser
import signal
import datetime, time

# Explicitly handle SIGINT and raise KeyboardInterrupt so that
# it's not handled by another thread
def sigint_handler(signum, frame):
    raise KeyboardInterrupt

def handle_sink(alert):
    global options
    content = alert.get_fields()[options.field][0].data
    now = datetime.datetime.now()
    outstamp = now.strftime("%Y%m%d.%H%M%S")
    setfilename = "%s.%s.set" %(options.file_prefix
                                and options.file_prefix
                                or options.field, outstamp)
    ipset = open(setfilename, "w")
    ipset.write(content)
    ipset.close()

def main():
    global options
    parser = OptionParser()
    parser.add_option("-c", "--channel", dest="channel",
                      help="snarf alert channel to subscribe to",
                      default="")
    parser.add_option("-f", "--file-prefix",
                      help="prefix for set filenames (defaults to field name)")
    (options, args) = parser.parse_args()

    if len(args) < 1:
        raise Exception("usage: %s field" %(sys.argv[0]))

    options.field=args[0]

    signal.signal(signal.SIGINT, sigint_handler)

    try:
        sink = Sink(handle_sink, channel=options.channel)
        while True:
            sink.get_thread().join(1000)
            if not sink.get_thread().isAlive:
                break
    except KeyboardInterrupt:
        sink.shutdown()

if __name__ == '__main__':
    main()
